% This .m file obtaines equations for second-order (five-parameter) process
% model identification from the characteristics areas (process models).
% Used in Section 3 of the paper.

%% Get equasions of the variables a1, a2, b1
close all
clear all

syms G K b1 b2 b3 b4 Tdm a1 a2 a3 a4
b2 = 0; b3 = 0; b4 = 0; a3 = 0; a4 = 0;

syms A0 A1 A2 A3 A4 A4_A0

% Equasions for area calculation from the process parameters
%A0 = K;
%A1 = A0*( a1 - b1 + Tdm );
%A2 = A0*( b2 - a2 - Tdm*b1 + (Tdm^2)/factorial(2) ) + A1*a1;
%A3 = A0*( a3 - b3 + Tdm*b2 - ((Tdm^2)*b1)/factorial(2) + (Tdm^3)/factorial(3) ) + A2*a1 - A1*a2;
%A4 = A0*( -a4 + b4 - Tdm*b3 + ((Tdm^2)*b2)/factorial(2) - ((Tdm^3)*b1)/factorial(3) + (Tdm^4)/factorial(4) ) + A3*a1 - A2*a2 + A1*a3;

eq = A1 == A0*( a1 - b1 + Tdm );
a1 = solve(eq, a1)

eq = A2 == A0*( b2 - a2 - Tdm*b1 + (Tdm^2)/factorial(2) ) + A1*a1;
a2 = solve(eq, a2)

eq = A3 == A0*( a3 - b3 + Tdm*b2 - ((Tdm^2)*b1)/factorial(2) + (Tdm^3)/factorial(3) ) + A2*a1 - A1*a2;
b1 = solve(eq, b1)

%% get equasion for the Tdm
clear all

syms b1 Tdm a1 a2
syms A0 A1 A2 A3 A4
syms result

%equasions solved in previous steps
b1 = -(- A0^3*Tdm^3 + 3*A0^2*A1*Tdm^2 + 6*A2*A0^2*Tdm + 6*A3*A0^2 - 6*A0*A1^2*Tdm...
    - 12*A2*A0*A1 + 6*A1^3)/(3*A0^3*Tdm^2 - 6*A0^2*A1*Tdm - 6*A2*A0^2 + 6*A0*A1^2);
a1 = (A1 + A0*b1 - A0*Tdm)/A0;
a2 = -(A2 + A0*(- Tdm^2/2 + b1*Tdm) - (A1*(A1 + A0*b1 - A0*Tdm))/A0)/A0;

eq = A4 == A0*( - ((Tdm^3)*b1)/factorial(3) + (Tdm^4)/factorial(4) ) + A3*a1 - A2*a2;
solve(eq, Tdm)
% A0^3*Tdm^6 - 6*A0^2*A1*Tdm^5 + 18*A0*A1^2*Tdm^4 - 6*A0^2*A2*Tdm^4 +
% 24*A0^2*A3*Tdm^3 - 24*A1^3*Tdm^3 - 144*A0*A1*A3*Tdm^2 + 72*A0^2*A4*Tdm^2 +
% 72*A1^2*A2*Tdm^2 + 144*A0*A2*A3*Tdm - 144*A0*A1*A4*Tdm + 144*A1^2*A3*Tdm -
% 144*A1*A2^2*Tdm - 288*A1*A2*A3 - 144*A0*A2*A4 + 144*A1^2*A4 + 144*A0*A3^2 + 144*A2^3

%% Collect terms for the Tdm
clear all
syms A0 A1 A2 A3 A4 Tdm G

G = A0^3*Tdm^6 - 6*A0^2*A1*Tdm^5 + 18*A0*A1^2*Tdm^4 - 6*A0^2*A2*Tdm^4 +...
    24*A0^2*A3*Tdm^3 - 24*A1^3*Tdm^3 - 144*A0*A1*A3*Tdm^2 + 72*A0^2*A4*Tdm^2 +...
    72*A1^2*A2*Tdm^2 + 144*A0*A2*A3*Tdm - 144*A0*A1*A4*Tdm + 144*A1^2*A3*Tdm -...
    144*A1*A2^2*Tdm - 288*A1*A2*A3 - 144*A0*A2*A4 + 144*A1^2*A4 + 144*A0*A3^2 + 144*A2^3;
G = collect(G,Tdm)

% For roots calculation
% A0^3*Tdm^6
% (-6*A0^2*A1)*Tdm^5
% (- 6*A2*A0^2 + 18*A0*A1^2)*Tdm^4
% (24*A3*A0^2 - 24*A1^3)*Tdm^3
% (72*A4*A0^2 - 144*A3*A0*A1 + 72*A2*A1^2)*Tdm^2
% (144*A3*A1^2 - 144*A1*A2^2 - 144*A0*A4*A1 + 144*A0*A3*A2)*Tdm
% 144*A4*A1^2 - 288*A1*A2*A3 + 144*A2^3 - 144*A0*A4*A2 + 144*A0*A3^2

%% Finding the correct Tdm of the second-order model using optimisation
clear all
close all
addpath('func_MAIN');
addpath('SubScripts');

function_type = "2DZ_TDM";

% Process parameters
n = 7;
Tdelay = 0.4894;
T = (1-Tdelay)/n; % T*n + Tdelay = 1
b = 0.03571;

num = [b 1];
den= 1;
den_1 = [T 1];
for a=1:n
	den = conv(den,den_1);
end 

% Time response of the Process
t = 0:0.01:10;
P_tf = tf(num,den,'InputDelay',Tdelay);
[p_res_original] = step(P_tf,t);  
[A0,A1,A2,A3,A4,A5] = numden2areas (num,den,Tdelay);

size1 = max(size(t));
Ts = (t(size1)-t(1))/(size1-1);
IAE_old = 1e100;

Tdelay_v = 0:1e-4:2;
disp('FInding correct Tdm. Please wait.');
for i=1:1:length(Tdelay_v)
    Tdm = Tdelay_v(i);
    disp(num2str(Tdm));
    
    if ( function_type == "2DZ_TDM" )
        b1 = -(- A0^3*Tdm.^3 + 3*A0^2*A1*Tdm.^2 + 6*A2*A0^2*Tdm + 6*A3*A0^2 - 6*A0*A1^2*Tdm ...
            - 12*A2*A0*A1 + 6*A1^3)./(3*A0^3*Tdm.^2 - 6*A0^2*A1*Tdm - 6*A2*A0^2 + 6*A0*A1^2);
        a1 = (A1 + A0*b1 - A0*Tdm)/A0;
        a2 = -(A2 + A0*(- Tdm.^2/2 + b1.*Tdm) - (A1*(A1 + A0*b1 - A0*Tdm))/A0)/A0;
        
        num=[A0*b1 A0];
        den=[a2 a1 1]; 
    else
        b1 = 0;
        a1 = A1/A0-Tdm;
        a2 = -A2/A0+A1*a1/A0+Tdm^2/2;
        
        num=[A0];
        den=[a2 a1 1];         
    end

    % Time response of the calculated model
    P_tf = tf(num,den,'InputDelay',Tdm);
    [p_res_calc] = step(P_tf,t);
    
    % IAE calculation
    err = p_res_original-p_res_calc;
    IAE = sum(abs(err))*Ts;
    IAE_out(i) = IAE;
    
    IAE_min = min(IAE_out);
    if IAE>IAE_min*100
        break;
    end
end

% Find minimum IAE - optimum solution
[M,I] = min(IAE_out);
Tdm = Tdelay_v(I);

% Calculate the rest of the model parameters from Tdm
if ( function_type == "2DZ_TDM" )
    b1 = -(- A0^3*Tdm.^3 + 3*A0^2*A1*Tdm.^2 + 6*A2*A0^2*Tdm + 6*A3*A0^2 - 6*A0*A1^2*Tdm ...
        - 12*A2*A0*A1 + 6*A1^3)./(3*A0^3*Tdm.^2 - 6*A0^2*A1*Tdm - 6*A2*A0^2 + 6*A0*A1^2);
    a1 = (A1 + A0*b1 - A0*Tdm)/A0;
    a2 = -(A2 + A0*(- Tdm.^2/2 + b1.*Tdm) - (A1*(A1 + A0*b1 - A0*Tdm))/A0)/A0;
    
    num=[A0*b1 A0];
    den=[a2 a1 1];
else
    b1 = 0;
    a1 = A1/A0-Tdm;
    a2 = -A2/A0+A1*a1/A0+Tdm^2/2;
    
    num=[A0];
    den=[a2 a1 1];
end
s = char(strcat('Tdelay b1 a1 a2',{newline},{'     '},num2str([Tdm b1 a1 a2])));
disp(s);

P_tf = tf(num,den,'InputDelay',Tdm);
[p_res_calc] = step(P_tf,t);

figure()
plot(t,p_res_original,'b','DisplayName','original');
hold on;
plot(t,p_res_calc,'--r','DisplayName','2D + Z + Tdelay');
ylim([min(p_res_original)*1.2, max(p_res_original)*1.2]);
legend show;
leg = legend('fontsize',10);
set(leg,'location','Best'); 

figure()
plot(Tdelay_v(1:i),IAE_out,'b','DisplayName','IAE');
legend show;
leg = legend('fontsize',10);
set(leg,'location','Best'); 