% Plot Bode plots of models -----------------------------------------------
nr_plots = size(y_data); nr_plots = nr_plots(2);

w_start = w_err(1)*10;
cond = find(w_start<=w_err);
w = w_err(cond(1):end);

mag_data = [];
phase_data = [];
for i=1:nr_plots
    [mag_P,phase_P] = bode(Ptf_data(i),w);
    mag_P = squeeze(mag_P); phase_P = squeeze(phase_P);
    [~,corrPhase,~] = bode(Ptf_data(i),0);
    phase_P = phase_P-corrPhase;     
    mag_data = [mag_data mag_P];
    phase_data = [phase_data phase_P];
end

[P_subplot,P_fig,P_colorbar] = subplot_position(400,200,1,2,2,5,8,25,12,50,50);
figure('Name',char(strcat('Process',{' '},display,{' '},'Bode plot')),'NumberTitle','off','Position',P_fig);

subplot('position',P_subplot{1});
for i=1:nr_plots
    [line_width,line_style] = get_line_width_style(i,method_compar(i));
    loglog(w,mag_data(:,i),eval(line_style),'linewidth',eval(line_width),'DisplayName',char(method_name(i)));
    hold on;
end
grid;
ah = gca;
set(ah,'box','off','color','none');
bh = axes('Position',get(ah,'Position'),'box','on','xtick',[],'ytick',[]);
axes(ah);
linkaxes([ah bh]);
xlim([w(1) w(end)]);
[min_nr,max_nr] = process_y_min_max(mag_data);
ylim([min_nr max_nr]);
set(gca,'XMinorTick','on','YMinorTick','on');
ylabel('Magnitude [/]');
set(gca,'xticklabel',[]);
leg = legend();
set(leg,'location','Best');

subplot('position',P_subplot{2});
for i=1:nr_plots
    [line_width,line_style] = get_line_width_style(i,method_compar(i));
    semilogx(w,phase_data(:,i),eval(line_style),'linewidth',eval(line_width),'DisplayName',char(method_name(i)));
    hold on;
end
grid;
ah = gca;
set(ah,'box','off','color','none');
bh = axes('Position',get(ah,'Position'),'box','on','xtick',[],'ytick',[]);
axes(ah);
linkaxes([ah bh]);
xlim([w(1) w(end)]);
[min_nr,~] = process_y_min_max(phase_data);
ylim([min_nr 0]);
set(gca,'XMinorTick','on','YMinorTick','on');
ylabel('Phase []');
xlabel('Frequency [rad/s]');

tightfigadv();

if ~exist(fullfile(pwd, image_dir), 'dir')
    mkdir(fullfile(pwd, image_dir));
end

img_name = strcat(char(image_name),'_',char(display),'_Bode');
export_figure_as_vecgra([], fullfile(pwd, image_dir, img_name), '-dmeta');
export_figure_as_vecgra([], fullfile(pwd, image_dir, img_name), '-dpdf');

function [min_nr,max_nr] = process_y_min_max(y_data)

min_nr = min(min(y_data));
max_nr = max(max(y_data));

if min_nr<0
    min_nr = min_nr*1.02;
else
    min_nr = min_nr*0.98;
end
max_nr = max_nr*1.02;

end