% Plot Bode plots of models under noise -----------------------------------

N_line_style = "'-'";
N_line_color = "'c'";
M_line_style = "'--'";
M_line_color = "'r'";
line_width = 0.5;
legend_text_size = 9;

nr_plots = length(Ptf_data);
w_st = w_err(1)*10;
cond = find(w_st<=w_err);
w_plot = w_err(cond(1):end);
w_plot = w_plot(1:1000:end);

mag_data = [];
phase_data = [];
for i=1:1:nr_plots
    [mag_P,phase_P] = bode(Ptf_data(i),w_plot);
    mag_P = squeeze(mag_P); phase_P = squeeze(phase_P);
    [~,corrPhase,~] = bode(Ptf_data(i),0);
    phase_P = phase_P-corrPhase;     
    mag_data = [mag_data mag_P];
    phase_data = [phase_data phase_P];
end


[P_subplot,P_fig,P_colorbar] = subplot_position(400,200,1,2,2,5,8,25,12,50,50);
figure('Name',char(strcat('Process',{' '},display,{' '},'Bode plot')),'NumberTitle','off','Position',P_fig);

subplot('position',P_subplot{1});
for i=1:1:nr_plots-1
    p1 = loglog(w_plot,mag_data(:,i),eval(N_line_style),'Color',eval(N_line_color),'linewidth',line_width);
    hold on;
end
p2 = loglog(w_plot,mag_data(:,nr_plots),eval(M_line_style),'Color',eval(M_line_color),'linewidth',line_width);
grid;
ah = gca;
set(ah,'box','off','color','none');
bh = axes('Position',get(ah,'Position'),'box','on','xtick',[],'ytick',[]);
axes(ah);
linkaxes([ah bh]);
xlim([w_plot(1) w_plot(end)]);
[min_nr,max_nr] = process_y_min_max(mag_data);
ylim([min_nr max_nr]);
set(gca,'XMinorTick','on','YMinorTick','on');
ylabel('Magnitude [/]');
set(gca,'xticklabel',[]);
leg = legend([p1(1),p2(1)],char(method),'Process','fontsize',legend_text_size);
leg.Position(3) = 0;
leg.Position(4) = 0;
set(leg,'location','Best');

subplot('position',P_subplot{2});
for i=1:1:nr_plots-1
p1 = semilogx(w_plot,phase_data(:,i),eval(N_line_style),'Color',eval(N_line_color),'linewidth',line_width);
    hold on;
end
p2 = semilogx(w_plot,phase_data(:,nr_plots),eval(M_line_style),'Color',eval(M_line_color),'linewidth',line_width);
grid;
ah = gca;
set(ah,'box','off','color','none');
bh = axes('Position',get(ah,'Position'),'box','on','xtick',[],'ytick',[]);
axes(ah);
linkaxes([ah bh]);
xlim([w_plot(1) w_plot(end)]);
[min_nr,~] = process_y_min_max(phase_data);
ylim([min_nr 0]);
set(gca,'XMinorTick','on','YMinorTick','on');
ylabel('Phase []');
xlabel('Frequency [rad/s]');

tightfigadv();

if ~exist(fullfile(pwd, image_dir), 'dir')
    mkdir(fullfile(pwd, image_dir));
end

img_name = strcat(char(image_name),'_',char(display),'_Bode_',num2str(n));
export_figure_as_vecgra([], fullfile(pwd, image_dir, img_name), '-dmeta');
export_figure_as_vecgra([], fullfile(pwd, image_dir, img_name), '-dpdf');

function [min_nr,max_nr] = process_y_min_max(y_data)

min_nr = min(min(y_data));
max_nr = max(max(y_data));

if min_nr<0
    min_nr = min_nr*1.02;
else
    min_nr = min_nr*0.98;
end
max_nr = max_nr*1.02;

end