% This .m file tests proposed model identification method in the
% time-domain (identification from an open-loop time response of a process).
% Used in Section 4.2 of the paper.

clear all;
close all;

% additionals folders
addpath('func_MAIN');
addpath('func_comparison');
addpath('SubScripts');
addpath('func_plot');
addpath('func_ExportFigure');

%image save
image_dir = '_Image_export_emf_';
image_name = 'Test_Tdomain';

% Simulation parameters
Ts = 0.001;

%% Comparison with various methods ----------------------------------------
display = {'Gp1'};

% Proces
num=[-0.2 1];den=[0.2 1.2 1];Tdelay=0.2;

% Simulation parameters
t_stop = 8;
tdomain_simulate_process;

% Broida -----------------------------------------------------------
[num,T,n,Tdelay,den] = Broida_model(t,y_process,u,t_step);
s = char(strcat('Broida (model-fitting) model parameters [Kpr T n Tdm]:',...
    {newline},{'     '},num2str([num T n Tdelay])));
disp(s);
method = 'Broida';
plot_compare = 'no';
tdomain_simulate_model;

% Astrom (time integral approach) ----------------------------------
[num,T,Tdelay,den] = Astrom_time_integral_model(t,y_process,t_step,A0,A1);
s = char(strcat('Astrm (time integral) model parameters [Kpr a1m a2m Tdm]:',...
    {newline},{'     '},num2str([num den(2) den(1) Tdelay])));
disp(s);
method = 'Astrm (time integral)';
plot_compare = 'no';
tdomain_simulate_model;

% Astrom (model-fitting approach) ----------------------------------
[num,T,Tdelay,den] = Astrom_fitting_model(t,y_process,t_step,A0,A1);
s = char(strcat('Astrm (model-fitting) model parameters [Kpr T Tdm]:',...
    {newline},{'     '},num2str([num T Tdelay])));
disp(s);
method = 'Astrm (model-fitting)';
plot_compare = 'no';
tdomain_simulate_model;

% Proposed method --------------------------------------------------
method = 'Proposed';
plot_compare = 'yes';

[Kpr,a1,a2,b1,Tdelay,error] = area2Model_2DZ_Tdm(A0,A1,A2,A3,A4,process_type);
if ( false == error(3) & false == error(4) )
    num = [Kpr*b1 Kpr];
    den = [a2 a1 1];
    tdomain_simulate_model;
    s = char(strcat('Proposed method model parameters [Kpr a1m a2m Tdm b1m]:',...
        {newline},{'     '},num2str([Kpr a1 a2 Tdelay b1])));
else
    s = char('Proposed method error!');
end
disp(s);

%Plots -------------------------------------------------------------
tdomain_print_criteria;
tdomain_plot_time_response
tdomain_plot_nyquist;
tdomain_plot_bode;

%% Comparison with various methods ----------------------------------------
display = {'Gp2'};

% Proces
num=[1]; den=[0.25 1]; Tdelay=0;
den = conv (den,den); den = conv (den,den); den = conv (den,den);

% Simulation parameters
t_stop = 7;
tdomain_simulate_process;

% Broida -----------------------------------------------------------
[num,T,n,Tdelay,den] = Broida_model(t,y_process,u,t_step);
s = char(strcat('Broida (model-fitting) model parameters [Kpr T n Tdm]:',...
    {newline},{'     '},num2str([num T n Tdelay])));
disp(s);
method = 'Broida';
plot_compare = 'no';
tdomain_simulate_model;

% Astrom (time integral approach) ----------------------------------
[num,T,Tdelay,den] = Astrom_time_integral_model(t,y_process,t_step,A0,A1);
s = char(strcat('Astrm (time integral) model parameters [Kpr a1m a2m Tdm]:',...
    {newline},{'     '},num2str([num den(2) den(1) Tdelay])));
disp(s);
method = 'Astrm (time integral)';
plot_compare = 'no';
tdomain_simulate_model;

% Astrom (model-fitting approach) ----------------------------------
[num,T,Tdelay,den] = Astrom_fitting_model(t,y_process,t_step,A0,A1);
s = char(strcat('Astrm (model-fitting) model parameters [Kpr T Tdm]:',...
    {newline},{'     '},num2str([num T Tdelay])));
disp(s);
method = 'Astrm (model-fitting)';
plot_compare = 'no';
tdomain_simulate_model;

% Proposed method --------------------------------------------------
method = 'Proposed';
plot_compare = 'yes';

[Kpr,a1,a2,b1,Tdelay,error] = area2Model_2DZ_Tdm(A0,A1,A2,A3,A4,process_type);
if ( false == error(3) & false == error(4) )
    num = [Kpr*b1 Kpr];
    den = [a2 a1 1];
    tdomain_simulate_model;
    s = char(strcat('Proposed method model parameters [Kpr a1m a2m Tdm b1m]:',...
        {newline},{'     '},num2str([Kpr a1 a2 Tdelay b1])));
else
    s = char('Proposed method error!');
end
disp(s);

%Plots -------------------------------------------------------------
tdomain_print_criteria;
tdomain_plot_time_response
tdomain_plot_nyquist;
tdomain_plot_bode;

%% Comparison with various methods ----------------------------------------
display = {'Gp3'};

% Proces
num=[-0.25 0.5];den=[2/100 29/100 1 1];Tdelay=0.7;

% Simulation parameters
t_stop = 10;
tdomain_simulate_process;

% Broida -----------------------------------------------------------
[num,T,n,Tdelay,den] = Broida_model(t,y_process,u,t_step);
s = char(strcat('Broida (model-fitting) model parameters [Kpr T n Tdm]:',...
    {newline},{'     '},num2str([num T n Tdelay])));
disp(s);
method = 'Broida';
plot_compare = 'no';
tdomain_simulate_model;

% Astrom (time integral approach) ----------------------------------
[num,T,Tdelay,den] = Astrom_time_integral_model(t,y_process,t_step,A0,A1);
s = char(strcat('Astrm (time integral) model parameters [Kpr a1m a2m Tdm]:',...
    {newline},{'     '},num2str([num den(2) den(1) Tdelay])));
disp(s);
method = 'Astrm (time integral)';
plot_compare = 'no';
tdomain_simulate_model;

% Astrom (model-fitting approach) ----------------------------------
[num,T,Tdelay,den] = Astrom_fitting_model(t,y_process,t_step,A0,A1);
s = char(strcat('Astrm (model-fitting) model parameters [Kpr T Tdm]:',...
    {newline},{'     '},num2str([num T Tdelay])));
disp(s);
method = 'Astrm (model-fitting)';
plot_compare = 'no';
tdomain_simulate_model;

% Proposed method --------------------------------------------------
method = 'Proposed';
plot_compare = 'yes';

[Kpr,a1,a2,b1,Tdelay,error] = area2Model_2DZ_Tdm(A0,A1,A2,A3,A4,process_type);
if ( false == error(3) & false == error(4) )
    num = [Kpr*b1 Kpr];
    den = [a2 a1 1];
    tdomain_simulate_model;
    s = char(strcat('Proposed method model parameters [Kpr a1m a2m Tdm b1m]:',...
        {newline},{'     '},num2str([Kpr a1 a2 Tdelay b1])));
else
    s = char('Proposed method error!');
end
disp(s);

%Plots -------------------------------------------------------------
tdomain_print_criteria;
tdomain_plot_time_response
tdomain_plot_nyquist;
tdomain_plot_bode;

%% Comparisson with step identification methods ---------------------------
display = {'Gp4'};
num = conv([-2.7*2.15 2.15],[158.5 6 1]);
den1 = [17.5 1]; den1 = conv(den1,den1); den1 = conv (den1,den1);
den = conv(den1,[20 1]);
Tdelay = 14;

% Simulation parameters
t_stop = 350;
tdomain_simulate_process;

% Three-point fitting ----------------------------------------------
% G. P. Rangaiah and P. R. Krishnaswamy, Estimating Second-Order plus Dead
% Time Model Parameters, Industrial & Engineering Chemistry Research,
% vol. 33, no. 7, pp. 18671871, Jul. 1994, doi: 10.1021/ie00031a029.
% Parameters published in T. Liu and F. Gao 2010
num = [2.15];
den = [2234.67 71.99 1];
Tdelay = 21.56;
method = 'Rangaiah';
s = char(strcat(method,{' '},'model parameters [Km a1m a2m Tdm b1m]:',...
    {newline},{'     '},num2str([num den(2) den(1) Tdelay 0])));
disp(s);
plot_compare = 'no';
tdomain_simulate_model;

% Time integral approach -------------------------------------------
% Q.-G. Wang and Y. Zhang, Robust identification of continuous systems
% with dead-time from step responses, Automatica, vol. 37, no. 3,
% pp. 377390, Mar. 2001, doi: 10.1016/S0005-1098(00)00177-1.
num = [2.15];
den = [1939.41 69.63 1];
Tdelay = 28.58;
method = 'Wang';
s = char(strcat(method,{' '},'model parameters [Km a1m a2m Tdm b1m]:',...
    {newline},{'     '},num2str([num den(2) den(1) Tdelay 0])));
disp(s);
plot_compare = 'no';
tdomain_simulate_model;

% Frequency response fitting ---------------------------------------
% T. Liu and F. Gao, A frequency domain step response identification method
% for continuous-time processes with time delay, Journal of Process Control,
% vol. 20, no. 7, pp. 800809, Aug. 2010, doi: 10.1016/j.jprocont.2010.04.007.
num = [2.1413];
den = [1903.12 70.975 1];
Tdelay = 27.96;
method = 'Liu 2010';
s = char(strcat(method,{' '},'model parameters [Km a1m a2m Tdm b1m]:',...
    {newline},{'     '},num2str([num den(2) den(1) Tdelay 0])));
disp(s);
plot_compare = 'no';
tdomain_simulate_model;

% Proposed method --------------------------------------------------
method = 'Proposed';
plot_compare = 'yes';

[Kpr,a1,a2,b1,Tdelay,error] = area2Model_2DZ_Tdm(A0,A1,A2,A3,A4,process_type);
if ( false == error(3) & false == error(4) )
    num = [Kpr*b1 Kpr];
    den = [a2 a1 1];
    tdomain_simulate_model;
    s = char(strcat('Proposed method model parameters [Kpr a1m a2m Tdm b1m]:',...
        {newline},{'     '},num2str([Kpr a1 a2 Tdelay b1])));
else
    s = char('Proposed method error!');
end
disp(s);

%Plots -------------------------------------------------------------
tdomain_print_criteria;
tdomain_plot_time_response
tdomain_plot_nyquist;
tdomain_plot_bode;

%% Comparisson with relay identification methods --------------------------
display = {'Gp5'};
num = [1];
den = conv([1 1],[10 1]);
Tdelay = 2;

% Simulation parameters
t_stop = 80;
tdomain_simulate_process;

% Describing function method ---------------------------------------
% W. Li, E. Eskinat, and W. L. Luyben, An improved autotune identification
% method, Industrial & Engineering Chemistry Research, vol. 30, no. 7,
% pp. 15301541, Jul. 1991, doi: 10.1021/ie00055a019.
num = [0.853];
den = conv([1.15 1],[7.416 1]);
Tdelay = 2;
method = 'Li';
s = char(strcat(method,{' '},'model parameters [Km a1m a2m Tdm b1m]:',...
    {newline},{'     '},num2str([num den(2) den(1) Tdelay 0])));
disp(s);
plot_compare = 'no';
tdomain_simulate_model;

% Curve fitting approach -------------------------------------------
% T. Liu, F. Gao, and Y. Wang, A systematic approach for on-line identification
% of second-order process model from relay feedback test, AIChE Journal,
% vol. 54, no. 6, pp. 15601578, Jun. 2008, doi: 10.1002/aic.11476.
num = [1];
den = conv([1.0073 1],[9.9921 1]);
Tdelay = 1.9991;
method = 'Liu 2008';
s = char(strcat(method,{' '},'model parameters [Km a1m a2m Tdm b1m]:',...
    {newline},{'     '},num2str([num den(2) den(1) Tdelay 0])));
disp(s);
plot_compare = 'no';
tdomain_simulate_model;

% Frequency response fitting ---------------------------------------
% V. Ramakrishnan and M. Chidambaram, Estimation of a SOPTD transfer function
% model using a single asymmetrical relay feedback test, Computers & Chemical Engineering,
% vol. 27, no. 12, pp. 17791784, Dec. 2003, doi: 10.1016/S0098-1354(03)00138-8.
num = [1.05];
den = conv([1.271 1],[9.766 1]);
Tdelay = 1.814;
method = 'Ramakrishnan';
s = char(strcat(method,{' '},'model parameters [Km a1m a2m Tdm b1m]:',...
    {newline},{'     '},num2str([num den(2) den(1) Tdelay 0])));
disp(s);
plot_compare = 'no';
tdomain_simulate_model;

% Proposed method --------------------------------------------------
method = 'Proposed';
plot_compare = 'yes';

[Kpr,a1,a2,b1,Tdelay,error] = area2Model_2DZ_Tdm(A0,A1,A2,A3,A4,process_type);
if ( false == error(3) & false == error(4) )
    num = [Kpr*b1 Kpr];
    den = [a2 a1 1];
    tdomain_simulate_model;
    s = char(strcat('Proposed method model parameters [Kpr a1m a2m Tdm b1m]:',...
        {newline},{'     '},num2str([Kpr a1 a2 Tdelay b1])));
else
    s = char('Proposed method error!');
end
disp(s);

%Plots -------------------------------------------------------------
tdomain_print_criteria;
tdomain_plot_time_response
tdomain_plot_nyquist;
tdomain_plot_bode;

%% Comparisson with relay identification methods --------------------------
display = {'Gp6'};
num = [-1 1];
den1 = [1 1];
den = conv(den1,den1); den = conv (den,den);
den = conv(den1,den);
Tdelay = 1;

% Simulation parameters
t_stop = 20;
tdomain_simulate_process;

% Describing function method ---------------------------------------
% S.-H. Shen, J.-S. Wu, and C.-C. Yu, Use of biased-relay feedback for
% system identification, AIChE Journal, vol. 42, no. 4, pp. 11741180,
% Apr. 1996, doi: 10.1002/aic.690420431.
% Parameters published in T. Liu and F. Gao 2010
num = [1];
den = [1.48 1]; den = conv(den,den);
Tdelay = 4.56;
method = 'Shen';
s = char(strcat(method,{' '},'model parameters [Km a1m a2m Tdm b1m]:',...
    {newline},{'     '},num2str([num den(2) den(1) Tdelay 0])));
disp(s);
plot_compare = 'no';
tdomain_simulate_model;

% Curve fitting approach -------------------------------------------
% I. Kaya and D. P. Atherton, Parameter estimation from relay autotuning
% with asymmetric limit cycle data, Journal of Process Control, vol. 11,
% no. 4, pp. 429439, Aug. 2001, doi: 10.1016/S0959-1524(99)00073-6.
num = [1];
den = [1.4767 1]; den = conv(den,den);
Tdelay = 4.125;
method = 'Kaya';
s = char(strcat(method,{' '},'model parameters [Km a1m a2m Tdm b1m]:',...
    {newline},{'     '},num2str([num den(2) den(1) Tdelay 0])));
disp(s);
plot_compare = 'no';
tdomain_simulate_model;

% Frequency response fitting ---------------------------------------
% T. Liu and F. Gao, A generalized relay identification method for time
% delay and non-minimum phase processes, Automatica, vol. 45, no. 4,
% pp. 10721079, Apr. 2009, doi: 10.1016/j.automatica.2008.11.024.
num = [0.9992];
den = [1.4781 1]; den = conv(den,den);
Tdelay = 4.037;
method = 'Liu 2009';
s = char(strcat(method,{' '},'model parameters [Km a1m a2m Tdm b1m]:',...
    {newline},{'     '},num2str([num den(2) den(1) Tdelay 0])));
disp(s);
plot_compare = 'no';
tdomain_simulate_model;

% Proposed method --------------------------------------------------
method = 'Proposed';
plot_compare = 'yes';

[Kpr,a1,a2,b1,Tdelay,error] = area2Model_2DZ_Tdm(A0,A1,A2,A3,A4,process_type);
if ( false == error(3) & false == error(4) )
    num = [Kpr*b1 Kpr];
    den = [a2 a1 1];
    tdomain_simulate_model;
    s = char(strcat('Proposed method model parameters [Kpr a1m a2m Tdm b1m]:',...
        {newline},{'     '},num2str([Kpr a1 a2 Tdelay b1])));
else
    s = char('Proposed method error!');
end
disp(s);

%Plots -------------------------------------------------------------
tdomain_print_criteria;
tdomain_plot_time_response
tdomain_plot_nyquist;
tdomain_plot_bode;
