function [Kpr,a1,a2,L,error_b] = area2Model2D (A0,A1,A2,A3);
% function [Kpr,a1,a2,L,error_b] = area2Model2D (A0,A1,A2,A3);
%
% Function calculates second-order process model parameters from
% characteristic areas.
%     Kpr*e-Tdm*s
% ---------------------
%   a2*s^2 + a1*s + 1
%
% INPUTS:
% A0-A3:        Characteristic areas.
%
% OUTPUTS:
% Kpr:      Process model steady-state gain.
% a1:       Process model time constant.
% a2:       Process model time constant.
% L:        Process model time-delay.
% error_b:	Errors binary (TRUE, FALSE) vector
%   error_b(1): L set to 0.
%   error_b(2): Process model not calculated.

    error_b = false(1, 2);

    Kpr = A0;

    Lv = roots([1/6,-A1/(2*Kpr),A1^2/Kpr^2-A2/Kpr,2*A1*A2/Kpr^2-A3/Kpr-A1^3/Kpr^3]);

    v1 = (abs(imag(Lv))<0.01*abs(real(Lv)));
    v2 = (imag(Lv) <1e-3*abs(A1/A0) );
    Lv = Lv( v1 | v2 );

    Lv = real(Lv);
    Lv = Lv(Lv >= 0);
    Lv( Lv<5e-4*abs(A1/A0) ) = 0; % Tdm too small
    L = min(Lv);

    for i=0:1:2

        if ( 0==length(L) | 0<i )
            L = 0;
            error_b(1) = true;      
        end

        a1 = A1/Kpr-L;
        if ( 2>i )
            a2 = -A2/Kpr+A1*a1/Kpr+L^2/2;
            if (abs(a2/a1^2) < 1e-6)
                a2 = 0;
            end
        else
            a2 = 0;
        end

        if (a1<0 | a2<0)
            error_b(2) = true;
            L = NaN;
            a1 = NaN;
            a2 = NaN;  
        else
            error_b(2) = false;
            break;
        end

    end

end