% function [A0,A1,A2,A3,A4,A5] = meas2areas2 (y,u,t,Tstart,Tinteg,Tavg,Tstop);
%
% function meas2areas.m calculates the characteristic areas
% from the process response and its first, second ... fifth integral.
%
% y - process response
% u - process input
% t - time vector
% Tstart - a start of experiment before the change of control variable (u)
%      (for the calculation of starting mean value)
% Tinteg - integration time
% Tinteg - a time when final mean value starts to calculate
% Tstop - a time till when the final mean value is calculated
%
% A0..A5 - areas calculated from the process response

function [A0,A1,A2,A3,A4,A5] = meas2areas2 (y,u,t,Tstart,Tinteg,Tavg,Tstop);

% Initialisation
a = max(size(y));
k = 1;

% Search for index of Tstart
while (t(k) < Tstart)
    k = k+1;
end

if (k > 1)
    kstart = k-1;
else
    kstart = k;
end

% Search for the change of the input variable (time instant and index)
u1 = u(kstart);
while (u(k) == u1) & (k < a)
    k = k+1;
end
kchange = k;
Tchange = t(kchange);

while (t(k) < Tinteg) & (k < a)
    k = k+1;
end
kinteg = k;
Tinteg = t(kinteg);

while (t(k) < Tavg) & (k < a)
    k = k+1;
end
kavg = k;

while (t(k) < Tstop) & (k < a)
    k = k+1;
end
kstop = k;

% Calculation of the mean values of the signals before change and after the process settles
ya0=mean(y(kstart:kchange-1));
ya1=mean(y(kavg:kstop));
ua0=mean(u(kstart:kchange-1));
ua1=mean(u(kavg:kstop));

DU = ua1-ua0;				% The amplitude of the process input change (u(inf)-u(0))

A0 = (ya1-ya0)/DU;		% Process gain
y = (y-ya0)/DU;			% Normalised response
u = (u-ua0)/DU;

dtv = t(kchange+1:kinteg+1)-t(kchange:kinteg);		% Sampling times

tx = t(kchange-1:kinteg)-t(kchange-1);
sys_int= tf([1],[1 0]);
% Calculation of the multiple integrals of the process input signal
Iu1 = lsim(sys_int,u(kchange-1:kinteg),tx);
Iu2 = lsim(sys_int,Iu1,tx);
Iu3 = lsim(sys_int,Iu2,tx);
Iu4 = lsim(sys_int,Iu3,tx);
Iu5 = lsim(sys_int,Iu4,tx);
% Calculation of the multiple integrals of the process output signal
Iy1 = lsim(sys_int,y(kchange-1:kinteg),tx);
Iy2 = lsim(sys_int,Iy1,tx);
Iy3 = lsim(sys_int,Iy2,tx);
Iy4 = lsim(sys_int,Iy3,tx);
Iy5 = lsim(sys_int,Iy4,tx);

% Calculation of areas
y1 = (A0*Iu1-Iy1);
A1 = y1(kinteg-kchange);
y2 = (A1*Iu1-A0*Iu2+Iy2);
A2 = y2(kinteg-kchange);
y3 = (A2*Iu1-A1*Iu2+A0*Iu3-Iy3);
A3 = y3(kinteg-kchange);
y4 = (A3*Iu1-A2*Iu2+A1*Iu3-A0*Iu4+Iy4);
A4 = y4(kinteg-kchange);
y5 = (A4*Iu1-A3*Iu2+A2*Iu3-A1*Iu4+A0*Iu5-Iy5);
A5 = y5(kinteg-kchange);
end
