function [Kpr,T,Td,den] = Astrom_fitting_model(t,y,t_ustep,A0,A1);
% [Kpr,T,Td,den] = Astrom_fitting_model(t,y,t_ustep,A0,A1)
%
% Function calculates first-order process model parameters from the process response (model fitting approach).
%
% K. J. strm and T. Hgglund, PID controllers : theory, design, and
% tuning, 2nd ed. Research Triangle Park, N.C.: International Society for
% Measurement and Control, 1995.
%
% INPUTS:
% t:        Time vector.
% y:        Process output vector.
% t_ustep:  Time at which process input changed.
% A0-A1:    Characteristic areas.
%
% OUTPUTS:
% Kpr:      Process model steady-state gain.
% T:        Process model time constant.
% Td:       Process model time-delay.
% den:      Process model denuminator.

Kpr = A0;
[tr,tl,iref] = findtrtl(t,y);
Td = tl - t_ustep;
Tar = A1/A0;
T = Tar - Td;

den = [T 1];

end