function [Kpr,T,Td,den] = Astrom_time_integral_model(t,y,t_ustep,A0,A1);
% function [Kpr,T,Td,den] = Astrom_time_integral_model(t,y,t_ustep,A0,A1)
%
% Function calculates second-order process model parameters from the process response (time-integral approach).
%
% K. J. strm and T. Hgglund, PID controllers : theory, design, and
% tuning, 2nd ed. Research Triangle Park, N.C.: International Society for
% Measurement and Control, 1995.
%
% INPUTS:
% t:        Time vector.
% y:        Process output vector.
% t_ustep:  Time at which process input changed.
% A0-A1:    Characteristic areas.
%
% OUTPUTS:
% Kpr:      Process model steady-state gain.
% T:        Process model time constant.
% Td:       Process model time-delay.
% den:      Process model denuminator.

Kpr = A0;
Tar = A1/A0;
F = griddedInterpolant(t,y); fun = @(t) F(t);
A1_a = integral(fun, 0, Tar+t_ustep);
T = (A1_a*(2.71828)^2)/(4*Kpr);
Td = Tar - 2*T;
if Td<0
    Td = 0;
end

den = [T^2 2*T 1];

end