function [K,Tbro,nbro,Td,den] = Broida_model(t,y,u,t_ustep);
% function [K,Tbro,nbro,Td,den] = Broida_model(t,y,u,t_ustep)
%
% Function calculates process model parameters from process response.
%
% J. Mikles and M. Fikar, Process Modelling, Identification, and Control,
% 1st ed. Berlin, Heidelberg: Springer Berlin Heidelberg, 2007.
%
% INPUTS:
% t:        Time vector.
% y:        Process output vector.
% u:        Process input vector.
% t_ustep:  Time at which process input changed.
%
% OUTPUTS:
% K:        Process model steady-state gain.
% Tbro:     Process model time constant.
% nbro:     Process model order.
% Td:       Process model time-delay.
% den:      Process model denuminator.

n=length(t);
if n<10000
    tn=t(1):1/(10*n):t(end);
else
    tn=t(1):0.1:t(end);
end
yn1=interp1(t,y,tn,'spline');
y=yn1;t=tn;
[b,i]=max(diff(y));
dti=t(i)-t(i-1);
b=b/dti;
ti=t(i);
yi=y(i);
K = (y(end) - y(1))/(u(end) - u(1));
a=yi-b*ti;
tu = -a/b;
tn = (K-a)/b - tu;

nfgstr = [ 1 2  3   4  5  6 ; 0 0.104   0.218   0.319   0.410   0.493; 1  0.368    0.271   0.224   0.195   0.161 ]';
nfgbro = [ 1 2  3   4  5  6; 0.000   0.096   0.192   0.268   0.331 0.385; 1.000   0.500   0.440   0.420   0.410   0.400]';
fs = tu/tn;
nbro = sum(nfgstr(:,2)<fs);
fn = nfgbro(nbro,2);
gn = nfgbro(nbro,3);
Tbro= gn * tn;
Td = ((fs-fn)*tn) - t_ustep;

den=1;
for i=1:nbro
  den=conv(den,[Tbro/i 1]);
end

if Td<0
    Td = 0;
end

end